@@ -4,45 +4,21 @@ from functools import wraps
 from pyfop.cache import cache
 
 
-def _isfop(val):
-    return isinstance(val, Aspect) or isinstance(val, PendingCall) or isinstance(val, Generator) or isinstance(val, Metamethod)
-
-
-class Generator:
-    def __init__(self, var):
-        assert isinstance(var, PendingCall)
-        self.method = var
-
-    def _call(self, context):
-        positional, unnamed = argparser.parse_positional(self.method.method, self.method.args)
-        kwargs = argparser.combine(argparser.parse_defaults(self.method.method),
-                                   positional,
-                                   self.method.kwargs)
-        for arg, val in kwargs.items():
-            if isinstance(arg, Aspect):
-                val.name = arg
-        # print(self.method.__name__, kwargs)
-        unnamed = [val._call(context) if _isfop(val) else val for val in unnamed]
-        kwargs = {arg: val._call(context) if _isfop(val) else val for arg, val in kwargs.items()}
-        return PendingCall(self.method.method, *unnamed, **kwargs, supercontext=context)
-
-    def call(self, **kwargs):
-        raise Exception("pyfop.Generator instances can only be passed to @lazy methods")
-
-    def _gather_aspects(self, context):
-        self.method._gather_aspects(context)
+def _isfop(val, inherits):
+    return isinstance(val, Aspect) or isinstance(val, PendingCall) or (isinstance(val, Metamethod) and val not in inherits)
 
 
 class PendingCall:
-    def __init__(self, _pyfop_method, *args, supercontext=None, **kwargs):
+    def __init__(self, _pyfop_method, *args, supercontext=None, inherits=(), **kwargs):
         self.method = _pyfop_method
         self.args = args
         self.kwargs = kwargs
         self.inject_aspects_to_context = dict()
-        self.supercontext = supercontext  # this is only to be set by the Generator class
+        self.supercontext = supercontext
+        self.inherits = inherits
 
     def __getattribute__(self, name):
-        if name in ["method", "args", "kwargs", "supercontext", "inject_aspects_to_context"] or name in dir(PendingCall):
+        if name in ["method", "args", "kwargs", "supercontext", "inject_aspects_to_context", "inherits"] or name in dir(PendingCall):
             return object.__getattribute__(self, name)
 
         def future_method(*args, fop_method_result, **kwargs):
@@ -102,10 +78,10 @@ class PendingCall:
         return lt(self, other)
 
     def __ge__(self, other):
-        return gt(self, other)
+        return ge(self, other)
 
     def __le__(self, other):
-        return lt(self, other)
+        return le(self, other)
 
     def __neg__(self):
         return neg(self)
@@ -125,11 +101,6 @@ class PendingCall:
         self._gather_aspects(context)
         return context
 
-    def get_input_methods(self):
-        methods = list()
-        self._gather_methods(methods)
-        return methods
-
     def call(self, **kwargs):
         context = Context()
         context.extend(kwargs, Priority.HIGH)
@@ -178,45 +149,18 @@ class PendingCall:
                 context.add(val.extended_name(), val, is_default=False)
         for val in kwargs.values():
             #print(self.method, val, isinstance(val, PendingCall))
-            if isinstance(val, PendingCall) or isinstance(val, Generator):
+            if isinstance(val, PendingCall):
                 val._gather_aspects(context)
-            if isinstance(val, Metamethod):
+            if isinstance(val, Metamethod) and _isfop(val, self.inherits):
                 from pyfop.utils import builder
                 builder(val.method)._gather_aspects(context)
         for val in self.inject_aspects_to_context.values():
-            if isinstance(val, PendingCall) or isinstance(val, Generator):
+            if isinstance(val, PendingCall):
                 val._gather_aspects(context)
-            if isinstance(val, Metamethod):
+            if isinstance(val, Metamethod) and _isfop(val, self.inherits):
                 from pyfop.utils import builder
                 builder(val.method)._gather_aspects(context)
 
-    def _gather_methods(self, methods):
-        methods.append(self.method._method if hasattr(self.method, "_method") else self.method)
-        defaults = argparser.parse_defaults(self.method)
-        positional, unnamed = argparser.parse_positional(self.method, self.args)
-        kwargs = argparser.combine(defaults, positional, self.kwargs)
-        for arg, val in defaults.items():
-            if isinstance(val, Aspect):
-                val.name = arg
-                if isinstance(kwargs[arg], Aspect):
-                    kwargs[arg].name = arg
-                kwargs[arg] = val
-        for arg, val in kwargs.items():
-            if isinstance(val, Aspect):
-                val.name = arg
-                if isinstance(kwargs[arg], Aspect):
-                    kwargs[arg].name = arg
-                kwargs[arg] = val
-        for arg, val in self.inject_aspects_to_context.items():
-            if isinstance(val, Aspect):
-                val.name = arg
-        for val in kwargs.values():
-            if isinstance(val, PendingCall):
-                val._gather_methods(methods)
-        for val in self.inject_aspects_to_context.values():
-            if isinstance(val, PendingCall):
-                val._gather_methods(methods)
-
     def _call(self, context):
         positional, unnamed = argparser.parse_positional(self.method, self.args)
         kwargs = argparser.combine(argparser.parse_defaults(self.method),
@@ -226,8 +170,8 @@ class PendingCall:
             if isinstance(val, Aspect):
                 val.name = arg
         # print(self.method.__name__, kwargs)
-        unnamed = [val._call(context) if _isfop(val) else val for val in unnamed]
-        kwargs = {arg: val._call(context) if _isfop(val) else val for arg, val in kwargs.items()}
+        unnamed = [val._call(context) if _isfop(val, self.inherits) else val for val in unnamed]
+        kwargs = {arg: val._call(context) if _isfop(val, self.inherits) else val for arg, val in kwargs.items()}
         return self.method(*unnamed, **kwargs)
 
 
@@ -236,13 +180,24 @@ class Metamethod:
         self.method = method
         self.supplementary_args = supplementary_args
         self.supplementary_kwargs = supplementary_kwargs
+        self.inherits = []
 
     def _call(self, context):
         from pyfop.utils import builder
         return builder(self.method)._call(context)
 
     def __call__(self, *args, **kwargs):
-        return PendingCall(self.method, *(self.supplementary_args+args), **(self.supplementary_kwargs | kwargs))
+        return PendingCall(self.method, *(self.supplementary_args+args), **(self.supplementary_kwargs | kwargs), inherits=self.inherits)
+
+
+def meta(*inherits):
+    def inner(method):
+        if not isinstance(method, Metamethod):
+            raise Exception("Can only wrap @lazy or @lazy_no_cache methods with @meta(...)")
+        method = Metamethod(method.method, *method.supplementary_args, **method.supplementary_kwargs)
+        method.inherits = set(inherits)
+        return method
+    return inner
 
 
 def lazy(method, *supplementary_args, **supplementary_kwargs):
@@ -342,7 +297,6 @@ def lt(x, y):
     return x < y
 
 
-
 @lazy
 def ge(x, y):
     return x >= y
